==> nest-infrastructure-spring-boot-5.5.0.1000-20251216.124241-1-sources.jar: com/digiwin/dap/nest/infrastructure/spring/boot/dapper/node/DwNodeSpanRestTemplate.java <==
package com.digiwin.dap.nest.infrastructure.spring.boot.dapper.node;

import com.digiwin.dap.nest.kernel.core.context.DwMapContextChain;
import com.digiwin.dap.nest.kernel.core.dapper.aspect.JaDapperAspectPointcut;
import com.digiwin.dap.nest.kernel.core.dapper.aspect.JaNodeSpanResolver;
import com.digiwin.dap.nest.kernel.core.dapper.log.DwLog;
import com.digiwin.dap.nest.kernel.core.dapper.log.DwMDC;
import com.digiwin.dap.nest.kernel.core.dapper.meta.NodeKind;
import com.digiwin.dap.nest.kernel.core.dapper.meta.NodeSpan;
import com.digiwin.dap.nest.kernel.core.util.datastructure.JaCollectionUtil;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringAopUtil;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringBeanUtil;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.aop.framework.ReflectiveMethodInvocation;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpRequest;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.ClientHttpRequestExecution;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.lang.reflect.Type;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

/**
 * TODO 未彻底监控, 短期解决方案
 * 未来需要重写RestTemplate.doExecute方法的字节码
 * (因需要创建byte[] class文件和需要javax tool.jar环境 且 k8s的busybox环境命令未知, 待验证)
 *
 * @author chenjian
 * @since 2024年05月29日 17:55:31
 */
@Component
@ConditionalOnClass(name = {DwNodeSpanRestTemplate.ProxyClassName})
public class DwNodeSpanRestTemplate implements JaNodeSpanResolver, JaDapperAspectPointcut, ClientHttpRequestInterceptor {

    public static final String ProxyClassName = "org.springframework.web.client.RestTemplate";

    static {
        initNodeKind();
    }

    @SuppressWarnings("rawtypes")
    private static void initNodeKind() {
        NodeKind.Constant.RestTemplate
                .buildRspHandler(o -> {
                    return o instanceof ResponseEntity ? ((ResponseEntity) o).getBody() : o;
                })
                .buildNodeIdHandler(invocation -> {
                    Object[] arguments = invocation.getArguments();
                    if (JaCollectionUtil.isNotEmpty(arguments)) {
                        Object argument = arguments[0];
                        if (argument instanceof String || argument instanceof URI) {
                            return argument.toString();
                        }
                        if (argument instanceof RequestEntity) {
                            return ((RequestEntity) argument).getUrl().toString();
                        }
                    }
                    return invocation.getMethod().hashCode() + "";
                })
                .buildReqArgsHandler(args -> {
                    if (JaCollectionUtil.isEmpty(args)) {
                        return args;
                    }
                    ArrayList<Object> objects = new ArrayList<>(args.length);
                    for (Object arg : args) {
                        if (arg instanceof String
                                || arg instanceof URI
                                || arg instanceof HttpMethod
                                || arg instanceof Type
                                || arg instanceof ParameterizedTypeReference
                        ) {
                            continue;
                        }
                        if (arg instanceof HttpEntity) {
                            objects.add(((HttpEntity) arg).getBody());
                        } else {
                            objects.add(arg);
                        }
                    }
                    return objects.toArray();
                })
                .buildSpanIdThreadLocal();
    }


    @Override
    public NodeSpan getNodeSpan(MethodInvocation invocation) {
        if (invocation instanceof ReflectiveMethodInvocation) {
            Object proxy = ((ReflectiveMethodInvocation) invocation).getProxy();
            if (proxy.toString().startsWith(ProxyClassName)) {
                return get(NodeKind.Constant.RestTemplate);
            }
        }
        return null;
    }

    @Override
    public String getExpression() {
//        return "execution(* org.springframework.web.client.RestTemplate.doExecute(..))";
        // org.springframework.web.client.RestOperations.postForEntity(java.lang.String, java.lang.Object, java.lang.Class<T>, java.lang.Object...)
        return "execution(* org.springframework.web.client.RestOperations.*(..))";
    }


    @NonNull
    @Override
    public ClientHttpResponse intercept(HttpRequest request, @NonNull byte[] body, ClientHttpRequestExecution execution) throws IOException {
        HttpHeaders headers = request.getHeaders();
        DwMapContextChain.getInstance().transmit().forEach(headers::add);
        headers.add(DwMDC.KeyParentSpanId, NodeKind.Constant.RestTemplate.transmitSpanId());
        return execution.execute(request, body);
    }

    @Override
    public void initNodeSpanResolver() {
        // 延迟依赖, 直接注入会导致aop失效
        List<RestTemplate> beanList = DwSpringBeanUtil.getBeanList(RestTemplate.class);
        if (JaCollectionUtil.isNotEmpty(beanList)) {
            beanList.forEach(restTemplate -> DwSpringAopUtil.getSingletonTarget(restTemplate).getInterceptors().add(0, this));
        } else {
            DwLog.warn("JaNodeSpanRestTemplate add interceptor error : restTemplate is null");
        }
    }
}

==> nest-infrastructure-spring-boot-5.5.0.1000-20251217.112142-5-sources.jar: com/digiwin/dap/nest/infrastructure/spring/boot/dapper/node/DwNodeSpanRestTemplate.java <==
package com.digiwin.dap.nest.infrastructure.spring.boot.dapper.node;

import com.digiwin.dap.nest.kernel.core.context.DwMapContextChain;
import com.digiwin.dap.nest.kernel.core.dapper.aspect.JaDapperAspectPointcut;
import com.digiwin.dap.nest.kernel.core.dapper.aspect.JaNodeSpanResolver;
import com.digiwin.dap.nest.kernel.core.dapper.log.DwLog;
import com.digiwin.dap.nest.kernel.core.dapper.log.DwMDC;
import com.digiwin.dap.nest.kernel.core.dapper.meta.NodeKind;
import com.digiwin.dap.nest.kernel.core.dapper.meta.NodeSpan;
import com.digiwin.dap.nest.kernel.core.util.datastructure.DwCollectionUtil;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringAopUtil;
import com.digiwin.dap.nest.infrastructure.spring.util.DwSpringBeanUtil;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.aop.framework.ReflectiveMethodInvocation;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpRequest;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.ClientHttpRequestExecution;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.lang.reflect.Type;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

/**
 * TODO 未彻底监控, 短期解决方案
 * 未来需要重写RestTemplate.doExecute方法的字节码
 * (因需要创建byte[] class文件和需要javax tool.jar环境 且 k8s的busybox环境命令未知, 待验证)
 *
 * @author chenjian
 * @since 2024年05月29日 17:55:31
 */
@Component
@ConditionalOnClass(name = {DwNodeSpanRestTemplate.ProxyClassName})
public class DwNodeSpanRestTemplate implements JaNodeSpanResolver, JaDapperAspectPointcut, ClientHttpRequestInterceptor {

    public static final String ProxyClassName = "org.springframework.web.client.RestTemplate";

    static {
        initNodeKind();
    }

    @SuppressWarnings("rawtypes")
    private static void initNodeKind() {
        NodeKind.Constant.RestTemplate
                .buildRspHandler(o -> {
                    return o instanceof ResponseEntity ? ((ResponseEntity) o).getBody() : o;
                })
                .buildNodeIdHandler(invocation -> {
                    Object[] arguments = invocation.getArguments();
                    if (DwCollectionUtil.isNotEmpty(arguments)) {
                        Object argument = arguments[0];
                        if (argument instanceof String || argument instanceof URI) {
                            return argument.toString();
                        }
                        if (argument instanceof RequestEntity) {
                            return ((RequestEntity) argument).getUrl().toString();
                        }
                    }
                    return invocation.getMethod().hashCode() + "";
                })
                .buildReqArgsHandler(args -> {
                    if (DwCollectionUtil.isEmpty(args)) {
                        return args;
                    }
                    ArrayList<Object> objects = new ArrayList<>(args.length);
                    for (Object arg : args) {
                        if (arg instanceof String
                                || arg instanceof URI
                                || arg instanceof HttpMethod
                                || arg instanceof Type
                                || arg instanceof ParameterizedTypeReference
                        ) {
                            continue;
                        }
                        if (arg instanceof HttpEntity) {
                            objects.add(((HttpEntity) arg).getBody());
                        } else {
                            objects.add(arg);
                        }
                    }
                    return objects.toArray();
                })
                .buildSpanIdThreadLocal();
    }


    @Override
    public NodeSpan getNodeSpan(MethodInvocation invocation) {
        if (invocation instanceof ReflectiveMethodInvocation) {
            Object proxy = ((ReflectiveMethodInvocation) invocation).getProxy();
            if (proxy.toString().startsWith(ProxyClassName)) {
                return get(NodeKind.Constant.RestTemplate);
            }
        }
        return null;
    }

    @Override
    public String getExpression() {
//        return "execution(* org.springframework.web.client.RestTemplate.doExecute(..))";
        // org.springframework.web.client.RestOperations.postForEntity(java.lang.String, java.lang.Object, java.lang.Class<T>, java.lang.Object...)
        return "execution(* org.springframework.web.client.RestOperations.*(..))";
    }


    @NonNull
    @Override
    public ClientHttpResponse intercept(HttpRequest request, @NonNull byte[] body, ClientHttpRequestExecution execution) throws IOException {
        HttpHeaders headers = request.getHeaders();
        DwMapContextChain.getInstance().transmit().forEach(headers::add);
        headers.add(DwMDC.KeyParentSpanId, NodeKind.Constant.RestTemplate.transmitSpanId());
        return execution.execute(request, body);
    }

    @Override
    public void initNodeSpanResolver() {
        // 延迟依赖, 直接注入会导致aop失效
        List<RestTemplate> beanList = DwSpringBeanUtil.getBeanList(RestTemplate.class);
        if (DwCollectionUtil.isNotEmpty(beanList)) {
            beanList.forEach(restTemplate -> DwSpringAopUtil.getSingletonTarget(restTemplate).getInterceptors().add(0, this));
        } else {
            DwLog.warn("JaNodeSpanRestTemplate add interceptor error : restTemplate is null");
        }
    }
}
